""" Twitter to text via template. Somehow that abbreviates to "twempest".
"""

# This file is part of Twempest.
#
# Twempest is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#
# Twempest is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with Twempest. If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2017 Dave Rogers <info@yukondude.com>

__version__ = "0.2.5"
